//============================================================================
// Hierarchical state machine manager
//
// Hkbd.C : Exemple of a state machine definition (keyboard manager exemple)
//
// (c) Robert Lacoste 2013, 
// Based on ideas from Miro Samek's book "Practival UML statecharts in C/C++)"
//
// Use this software freely, but provided without any warranty or support
//============================================================================

#include <stdio.h>
#include <stdlib.h>
#include "Timer.h"
#include "Hkbd.h"
#include "Hcmd.h"

//----------------------------------------------------------------------------
// Applicative constant and variables
//----------------------------------------------------------------------------

// Name of this specific HSM
#define HSM_NAME Hkbd

#define CODELEN 4
#define CODEREF "12AC"

unsigned short Len;
char Code[CODELEN];


//----------------------------------------------------------------------------
// State machine initialisation
//----------------------------------------------------------------------------

HSMINIT(HSM_PRIORITY_LOW,WaitingActivation)


//----------------------------------------------------------------------------
// State handlers (one per state)
//----------------------------------------------------------------------------

STATE(WaitingActivation)
		WHENENTRY
			DO
				Len=0;
			END
		ENDEVENT
		WHENEXIT
			DO
				TimerArm(TIMER_T1,HSM_ID_Hkbd,10000L);
			END
		ENDEVENT
		WHENEVENT(Hkbd_KEY_ACTIVATE)
			DO
				printf("Enter code : \n");
				NEWSTATE(WaitingDigits)
			END
		ENDEVENT
ENDSTATE

STATE(WaitingDigits)
		WHENENTRY
			DO
				printf("(digit)");
			END
		ENDEVENT
		WHENEVENT(Hkbd_KEY_ALPHAMODE)
			DO
				NEWSTATE(WaitingAlpha)
			END
		ENDEVENT
		WHENEVENT(Hkbd_KEY_DIGIT)
			GUARD(Len<CODELEN)
				DO
					putch(Code[Len++]=(unsigned char)Signal.SignalParameter+'0');
				END
			ENDGUARD
		ENDEVENT
ENDSTATE_WITH_MACROSTATE(MacroStateDigits)


STATE(WaitingAlpha)
		WHENENTRY
			DO
				printf("(alpha)");
			END
		ENDEVENT
		WHENEVENT(Hkbd_KEY_ALPHAMODE)
			DO
				NEWSTATE(WaitingDigits)
			END
		ENDEVENT
		WHENEVENT(Hkbd_KEY_DIGIT)
			GUARD(Len<CODELEN)
				DO
					putch(Code[Len++]=(unsigned char)Signal.SignalParameter+'A');
					NEWSTATE(WaitingDigits)
				END
			ENDGUARD
		ENDEVENT
ENDSTATE_WITH_MACROSTATE(MacroStateDigits)
	

STATE(MacroStateDigits)
		WHENEVENT(TIMER_TIMEOUT_T1)
			DO
				printf("Redo from start\n");
				NEWSTATE(WaitingActivation)
			END
		ENDEVENT
		WHENEVENT(Hkbd_KEY_DEL)
			GUARD(Len>0)
				DO
					printf("(del)");
					Len--;
				END
			ENDGUARD
		ENDEVENT
		WHENEVENT(Hkbd_KEY_ENTER)
			GUARD((Len==CODELEN)&&(strncmp(Code,CODEREF,Len)==0))	
				DO
					HsmSendMsg(HSM_ID_Hcmd,Hcmd_GIVE_SOME,0);
					NEWSTATE(WaitingSafetyDelay)
				END
			ELSE
				DO
					printf("Sorry !\n");
					NEWSTATE(WaitingActivation)
				END
			ENDGUARD
		ENDEVENT
ENDSTATE


STATE(WaitingSafetyDelay)
		WHENENTRY	
			DO
				TimerArm(TIMER_T2,HSM_ID_Hkbd,30000L);
			END
		ENDEVENT
		WHENEXIT
			DO
				printf("Next one, please...\n");
			END
		ENDEVENT
		WHENEVENT(TIMER_TIMEOUT_T2)
			DO
				NEWSTATE(WaitingActivation)
			END
		ENDEVENT
ENDSTATE

// END OF FILE



