//============================================================================
// Hierarchical state machine manager
//
// HSM.h : System level header file
//
// (c) Robert Lacoste 2013, 
// Based on ideas from Miro Samek's book "Practival UML statecharts in C/C++)"
//
// Use this software freely, but provided without any warranty or support
//============================================================================

#ifndef HSM_H
#define HSM_H

// System wide constants
#define HSM_PER_SYSTEM 5
#define HSM_MSG_QUEUE_LENGTH 10
#define HSM_MAX_SIGNAL_PER_HSM 200

// Internal constants
#define HSM_ID_NOTAFFECTED 0xFF
#define GENERIC_SIGNALIDSTART (HSM_MAX_SIGNAL_PER_HSM+1)
typedef enum {HSM_ON_ENTRY=GENERIC_SIGNALIDSTART,HSM_ON_EXIT} T_ID_SIGNALGENERIC;

// HSM types
typedef unsigned char HSM_T_HSMID;
typedef unsigned char HSM_T_SIGNALID;
typedef unsigned long HSM_T_SIGNALPARAMETER;

typedef struct 
{
	HSM_T_SIGNALID SignalId;
	HSM_T_SIGNALPARAMETER SignalParameter;
} HSM_T_SIGNAL;

typedef enum {HSM_PRIORITY_HIGH=0, HSM_PRIORITY_MEDIUM, HSM_PRIORITY_LOW, HSM_UNUSED} HSM_T_PRIORITY;

typedef void (* HSM_T_STATEHANDLINGFUNCTION)(const HSM_T_SIGNAL Signal) ;
typedef void (* HSM_T_IDLEFUNCTION)(void) ;

// HSM global variables
extern HSM_T_SIGNAL HsmSignalQueue[HSM_PER_SYSTEM][HSM_MSG_QUEUE_LENGTH];
extern unsigned short HsmSignalQueueFirstFreePos[HSM_PER_SYSTEM];
extern unsigned short HsmSignalQueueFirstUsedPos[HSM_PER_SYSTEM];
extern HSM_T_STATEHANDLINGFUNCTION HsmCurrentState[HSM_PER_SYSTEM];
extern HSM_T_PRIORITY HsmPriority[HSM_PER_SYSTEM];

// Prototype declarations
void HsmInit(void);
void HsmDefineIdleFunction(HSM_T_IDLEFUNCTION idle_function);
void HsmSetNewState(HSM_T_HSMID hsm_id,HSM_T_STATEHANDLINGFUNCTION NewState);
void HsmSendMsg(HSM_T_HSMID hsm_id,HSM_T_SIGNALID signalid,HSM_T_SIGNALPARAMETER parameter);
void HsmRun(void);
	
#endif

// END OF FILE


